#include <oxstd.h>
#include <oxfloat.h>
#import <lib/longrun>
#import <packages/dpd/dpd>

class DPDEx : DPD
{
	DPDEx();
	GetPrint();
	virtual GetLongRunNames();
	virtual GetLongRunInfo();
	TestRestrictions(mR, ...);
};
DPDEx::DPDEx()
{
	DPD();
}
DPDEx::GetPrint()
{
	return m_fPrint;
}
DPDEx::GetLongRunNames()
{
	decl as, m0, m1, m3, mlrcoef, mlrvar, vsely, vselx, asnames = Database::GetAllNames();

	[m0, m1, mlrcoef, m3, vsely, vselx, mlrvar] = ::LongRun(GetSelInfo(), Y_VAR,
			X_VAR, GetPar()', 0, 0, FALSE, asnames);
	return { array(asnames[vselx]) ~ GetDummyNames(), mlrcoef, mlrvar };
}
DPDEx::GetLongRunInfo()
{
	decl as, m0, m1, m3, mlrcoef, mlrvar, vsely, vselx, asnames = Database::GetAllNames();

	[m0, m1, mlrcoef, m3, vsely, vselx, mlrvar] = ::LongRun(GetSelInfo(), Y_VAR,
			X_VAR, GetPar()', GetCovar(), GetResVar(), FALSE, asnames);
	return { array(asnames[vselx]) ~ GetDummyNames(), mlrcoef', mlrvar };
}
DPDEx::TestRestrictions(mR, ...)
{
    decl cr, vr, msel, test, ctdf = 0, as, bsubset, blongrun = FALSE,
		mrtr, vrbr, pval;
    decl args = va_arglist();

	if (sizeof(args) == 0 || sizerc(args[0]) == 0)
    {
        msel = mR;
        mR = unit(sizerc(mR))[vecindex(msel)][]; // mR selects vars for exclusion
        bsubset = TRUE;                          // translate to selection matrix
        cr = sizer(mR);
        vr = zeros(cr, 1);
    }
    else
    {   bsubset = FALSE;
        cr = sizer(mR);
        vr = vec(args[0]);
        if (sizerc(vr) != cr)
        {
            println("Need: rows(mR) == sizerc(vR)");
            return;
        }
    }
    if (sizeof(args) > 1 && args[1] == 1)
    {
        blongrun = TRUE;
        [as, vrbr, mrtr] = GetLongRunInfo();
    }
    else
    {   vrbr = GetFreePar();
        mrtr = GetCovar();
        as = GetFreeParNames();
    }
	if (sizec(mR) != sizec(mrtr))
	{
		println("Warning: extending restrictions to exclude:", as[sizec(mR) : ]);
		mR ~= zeros(sizer(mR), sizec(mrtr) - sizec(mR));
	}

	mrtr = invertgen(mR * mrtr * mR', 30);        /* mrtr = R * Var(b) * R' */
    vrbr = mR * vrbr - vr;                                 /* vrbr = Rb - r */
    test = double(vrbr' * mrtr * vrbr);     /* (Rb-r)' * inv(mrtr) * (Rb-r) */
        
	if (m_fPrint)
	{
		if (bsubset)
	    {   println("\nTest for excluding", blongrun ? " from the long run" : "",
	            ": ", as[vecindex(msel)]);
	        PrintTestVal(test, cr, ctdf, "Subset");
	    }
	    else
	    {   println("\nTest for linear restrictions (Rb=r)", blongrun ? " on the long run" : "",
	            ":");
	        println("R matrix", "%c", as, mR, "r vector", vr');
	        PrintTestVal(test, cr, ctdf, "LinRes");
	    }
	}
	
	if (ctdf > 0)
        pval = tailf(test, cr, ctdf);
    else
    	pval = tailchi(test, cr);
	
	return test | pval;
}


DpdRecur(const dpd, const sYear, const iT0, const iMethod,
	const iTransform, const cStep, const bLongRun, const sFileName, ...)
{
    //-------------- recursive estimation  ----------------
	decl year, useyear, tmin, tmax, i, j, vselx;
	decl vtime, vb, vbse, vrss, vr2, vblr, vblrse;
    decl args = va_arglist(), clrtest = 0, avblrtest = {}, amr, avr, asr;
	decl bprint = dpd.GetPrint();

	if (sizeof(args) > 1)
	{
		// allow for an array of tests
		if (isarray(args[0]))
			amr = args[0];
		else
			amr = array(args[0]);
		if (isarray(args[1]))
			avr = args[1];
		else
			avr = array(args[1]);
		clrtest = sizeof(amr);

		asr = avblrtest = new array[clrtest];
		for (j = 0; j < clrtest; ++j)
		{
			avblrtest[j] = <>;
			if (sizeof(args) > 2 && isarray(args[2]))
				asr[j] = args[2][j];
			else
				asr[j] = sprint("test ", j);
		}
	}
	
	year = dpd.GetVar(sYear);
	tmin = int(min(deleter(year)));
	tmax = int(max(deleter(year)));

	vtime = vb = vbse = vrss = vr2 = vblr = vblrse = <>;

	tmin += iT0;	 // don't start completely at beginning 

	for (i = tmin; i <= tmax; ++i)		  // loop over time
	{
		useyear = !isdotnan(year) .&& year .<= i .? year .: M_NAN;// drop years
		dpd.Renew(useyear, sYear); // replace year variable
			  // respecify columns with years; must be done
		dpd.SetYear(sYear);// each time year var is changed

		dpd.SetTransform(iTransform);       // reloads data
	    dpd.SetMethod(iMethod);

		if (i == tmax)
			dpd.SetPrint(TRUE);// switch on standard output
		for (j = 1; j <= cStep; ++j)
	    	dpd.Estimate();            // 1-step estimation
		if (i == tmax)
			dpd.SetPrint(bprint);                  // reset
		
		vb ~= dpd.GetPar();	     // extract estimation info
		vbse ~= sqrt(diagonal(dpd.GetCovar())');
		vrss ~=	dpd.GetRSS();
		vr2 ~=	dpd.GetR2();
		vtime ~= i;

		if (bLongRun)
		{
			decl mp0, mlrcov, mlrun, mselr, vsely;
			[ mp0, mlrcov, mlrun, mselr, vsely, vselx ] = LongRun(
				dpd.GetSelInfo(), Y_VAR, X_VAR, dpd.GetPar()',
				dpd.GetCovar(), sqr(dpd.GetSigma()), FALSE,
				dpd.GetAllNames());
			vblr ~= mlrun';
			vblrse ~= mselr';

			for (j = 0; j < clrtest; ++j)
				avblrtest[j] ~= dpd.TestRestrictions(amr[j], avr[j], 1);
		}
	}
	dpd.Renew(year, sYear);// reset original year variable
	decl asw = dpd.GetRegNames(X_VAR), astw = {"year"} ~ asw, aslr;
	
	print("Coefficients", "%c", astw, vtime' ~ vb',
		  "Coefficient std.errors", "%c", astw, vtime' ~ vbse',
		  "%c", {"year", "RSS", "R^2"}, vtime' ~ vrss' ~ vr2');

	if (bLongRun)
	{
		aslr = array(dpd.GetAllNames()[vselx]) ~ dpd.GetDummyNames();
		
		print("Long-run coefficients", "%c",
			  {"year"} ~ aslr, vtime' ~ vblr',
		  	  "Long-run coefficient std.errors", "%c",
			  {"year"} ~ aslr, vtime' ~ vblrse');
		if (clrtest)
		{
			for (j = 0; j < clrtest; ++j)
			{
				print("Long-run test: ", asr[j], "%c",
					  {"year", sprint("LRtest", j), sprint("LRpval",j)},
					  	vtime' ~ avblrtest[j]');
		        println("R matrix", "%c", aslr, amr[j], "r vector", avr[j]');
			}
		}
		
	}
		  
	// save as .in7 file for easy graphing in GiveWin
	if (isstring(sFileName))
	{
		decl db = new Database(), aswse = asw;
		for (i = 0; i < sizeof(aswse); ++i)
			aswse[i] = "SE-" ~ aswse[i];
			
		db.Create(1, tmin, 1, tmax, 1);
		db.Append(vb' ~ vbse' ~ vrss' ~ vr2',
			asw ~ aswse ~ {"RSS", "R^2"});
		if (bLongRun)
		{
			decl aslrse = {};

			for (i = 0; i < sizeof(aslr); ++i)
			{	aslr[i] = "LR-" ~ aslr[i];
				aslrse ~= "SELR-" ~ aslr[i];
			}			
			db.Append(vblr' ~ vblrse',	aslr ~ aslrse);
			for (j = 0; j < clrtest; ++j)
			{
				db.Append(avblrtest[j]', {sprint("LRtest", j), sprint("LRpval",j)});
			}
		}
		db.SaveIn7(sFileName);
		delete db;
	}
}

